function bits_out = RLF_encode(bits_in, block, blocks_needed, key)
%
% bits_out = RLF_encode(bits_in, block, blocks_needed, key)
%
% Implementation of the random linear fountain encoder. 
% 
% Input Parameters:
%
%   - bits_in - input bitstream, vector of 0 and 1
%
%   - block   - symbol length
%
%   - blocks_needed  - number of desired symbols of the encoded stream
%
%   - key - integer, the seed for the PRNG
%
% Output:
% 
% bits_out - encoded stream, vector of 0 and 1
%
% ------------------------------------------------------------------------
%
% Copyright (C) 2011 Paweł Korus <pkorus@agh.edu.pl>
%
% Version: 2011.12.16
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation version 3.0.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%

    % Pad with 0's
    if mod(length(bits_in), block) ~= 0
        pad = zeros(1, block - mod(length(bits_in), block));
        bits_in = [bits_in pad];
    end
    
    % Create a matrix for the resulting bit-stream
    bits_out = false(1, block*blocks_needed);
    
    % Initialize the PRNG
    rand('twister', key);
    
    for i = 1:blocks_needed
        ro = randint(1, length(bits_in)/block, 2); 
        P = find(ro);
        B = zeros(numel(P), block);
        for b = 1:numel(P)
            bi = P(b);
            B(b, :) = bits_in(1+(bi-1)*block:bi*block);
        end
        if numel(P) == 1
            bits_out(1+(i-1)*block:i*block) = mod(B, 2);
        else
            bits_out(1+(i-1)*block:i*block) = mod(sum(B), 2);
        end
    end
end
